EnchantingDeviceConfiguration subclass: #NXTConfiguration
	instanceVariableNames: 'sensors motors pilot '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Enchanting-Configuration'!
!NXTConfiguration commentStamp: '<historical>' prior: 0!
NXTConfiguration contains all the information necessary to configure a LEGO Mindstorms NXT device (by specifying which sensors are in use and where they are attached.)!


!NXTConfiguration methodsFor: 'accessing' stamp: 'CWB 12/15/2010 22:15'!
configuredPeripherals
"Returns a list of all the configured peripherals"
	| list |

	list _ OrderedCollection new.
	list addAll: motors.
	list addAll: sensors.
	list add: pilot.
	"Might add things that are always configured here."

	list _ list reject: [:item | item isNil].
	^ list.
! !

!NXTConfiguration methodsFor: 'accessing' stamp: 'CWB 5/25/2011 21:04'!
setOfConfiguredClasses
"Returns a set of all the configured classes"

	| list |

	list _ OrderedCollection new.
	list _ self configuredPeripherals collect: [:item | item class].
	list add: NXTButtons.	"There is always one set of buttons (under the display)."
	^ list asSet.! !


!NXTConfiguration methodsFor: 'dialog interaction' stamp: 'CWB 1/1/2011 20:44'!
applyChangesFrom: dialog named: dialogName over: scratchFrame
"The user pressed okay on the dialog; update structures to reflect the changes."

	| destArray nameMapping |

	destArray _ nil.

	"In this map, we will take previous names and use them as keys, setting all values to nil.  Then we will look through all peripherals, put their original name in as a key, and their new name in as a value.  Anything that still has 'nil' as a value has been deleted."
	nameMapping _ Dictionary new.

	(dialogName = 'sensors') ifTrue: [ destArray _ sensors ].
	(dialogName = 'motors') ifTrue: [ destArray _ motors ].
	(dialogName = 'pilot') ifTrue: [ destArray _ OrderedCollection with: pilot ].

	destArray ifNil: [ self error: 'Unknown configuration dialog' ].

	"Get the previous names"
	destArray do: [:p |
		p ifNotNil: [ nameMapping at: (p name) put: nil ].
	].


	(1 to: destArray size) do: [ :index |
		| p |
		p _ dialog peripheralAt: index.
		destArray at: index put: p.
		p ifNotNil: [ 
			| configMorph |
			p portIndex: index.
			configMorph _ dialog blockAt: index.
			nameMapping at: (configMorph initialName) put: (p name).
		 ].
	].

	(dialogName = 'pilot') ifTrue: [ pilot _ destArray first ].

	self replaceNamesInScriptsUsing: nameMapping over: scratchFrame.

! !

!NXTConfiguration methodsFor: 'dialog interaction' stamp: 'CWB 12/4/2010 11:17'!
portsFor: dialogName
"Returns an array containing:
	an array with the names of all the ports
	an array (of the same size) of ConfigurationBlockMorphs (or nil) for each port."

	| sourceArray baseName count countMethod portNames portBlocks |

	sourceArray _ nil.

	(dialogName = 'sensors') ifTrue: [
		sourceArray _ sensors.
		baseName _ 'Sensor Port'.
		countMethod _ #numeric.		
	].
	(dialogName = 'motors') ifTrue: [
		sourceArray _ motors.
		baseName _ 'Motor Port'.
		countMethod _ #alphabetical.		
	].
	(dialogName = 'pilot') ifTrue: [
		sourceArray _ OrderedCollection with: pilot.
		baseName _ 'Pilot Type'.
		countMethod _ #none.		
	].

	portNames _ OrderedCollection new.
	portBlocks _ OrderedCollection new.

	sourceArray ifNil: [ self beep. ^ { portNames . portBlocks } ].

	count _ 1.
	sourceArray do: [ :peripheral |
		portNames add: (self class portName: baseName at: count method: countMethod).
		portBlocks add: (peripheral ifNotNil: [peripheral createConfigBlock] ifNil: [nil]).
		count _ count + 1.
	].
		

	^ { portNames . portBlocks }! !


!NXTConfiguration methodsFor: 'initialization' stamp: 'CWB 12/17/2010 15:08'!
initialize

	super initialize.
	motors _ OrderedCollection with: nil with: nil with: nil.
	sensors _ OrderedCollection with: nil with: nil with: nil with: nil.
	pilot _ nil.
! !


!NXTConfiguration methodsFor: 'object i/o' stamp: 'CWB 12/7/2010 20:35'!
fieldsVersion

	^ 1! !

!NXTConfiguration methodsFor: 'object i/o' stamp: 'CWB 12/7/2010 20:40'!
initFieldsFrom: anObjStream version: classVersion

	super initFieldsFrom: anObjStream version: classVersion.
	self initFieldsNamed: #(
		sensors
		motors
		pilot
	) from: anObjStream.! !

!NXTConfiguration methodsFor: 'object i/o' stamp: 'CWB 12/7/2010 20:38'!
storeFieldsOn: anObjStream

	super storeFieldsOn: anObjStream.
	self storeFieldsNamed: #(
		sensors
		motors
		pilot
	) on: anObjStream.! !


!NXTConfiguration methodsFor: 'other' stamp: 'CWB 11/21/2011 17:29'!
customExportColor: aColor for: aBlock on: aStream with: anExporter

"Outputs something like 'EObject.createList(240.0f, 12.0f, 38.0f)' where the numbers are:
  hue -- the color, 0 to 360 (degrees),
  saturation -- how true/vivid the colour is, 0 to 100 (%),
  brightness -- how bright the colour is, 0 to 100 (%)"

	aStream nextPutAll: 'EObject.createList('.
	anExporter exportLiteralValue: ((aColor hue) roundTo: 0.1) on: aStream.
	aStream nextPutAll: ', '.
	anExporter exportLiteralValue: ((aColor saturation * 100) roundTo: 0.1) on: aStream.
	aStream nextPutAll: ', '.
	anExporter exportLiteralValue: ((aColor brightness * 100) roundTo: 0.1) on: aStream.
	aStream nextPutAll: ')'.


! !

!NXTConfiguration methodsFor: 'other' stamp: 'MJM 3/29/2011 18:43'!
uncoloredArgMorphFor: specString

	"Allows the NXT device to create an argument other than that created by CommandBlockMorph."

	"Returns nil or an argMorph."

	| code |
	
	code _ specString at: 2.

	$I = code ifTrue: [^ ExpressionArgMorphWithMenu new numExpression: '1';
	menuSelector: #nxtInstrumentMenu].

	^ nil.
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NXTConfiguration class
	instanceVariableNames: ''!

!NXTConfiguration class methodsFor: 'dialog interaction' stamp: 'CWB 1/1/2011 15:28'!
blockColorFor: dialogName
"Returns the color ConfigurationBlockMorphs should be in the specified dialog"
	
	| mapping category |

	mapping _ Dictionary newFrom: {'sensors' -> 'sensing'. 'motors' -> 'motion'.}.
	category _ mapping at: dialogName ifAbsent: [ dialogName ].	

	^ super blockColorFor: category.
! !

!NXTConfiguration class methodsFor: 'dialog interaction' stamp: 'CWB 11/24/2011 06:58'!
orderedPeripheralClasses

	"Returns a list of all peripherals that might be set up for this device.
	The order is important, as it will be followed when placing blocks in the
	palettes in the main user interface.  (In other words, if you return 
	{ WhizBang . SawBuzz } the WhizBang blocks will always appear before
	the SawBuzz blocks."

	^ { 
		NXTDCMotor . 
		NXTMotor .
		NXTButtons .
		NXTTouchSensor . 
		NXTRangeDetector . 
		NXTLightDetector . 
		NXTSoundSensor .
		NXTCompassSensor .
		NXTGyroSensor .
	}! !

!NXTConfiguration class methodsFor: 'dialog interaction' stamp: 'CWB 11/24/2011 06:58'!
peripheralsFor: dialogName
"Returns a list of all the peripherals for the specified dialog"
	
	(dialogName = 'sensors') ifTrue: [
		^ {NXTLightDetector . NXTRangeDetector . NXTTouchSensor . NXTSoundSensor . NXTCompassSensor . NXTGyroSensor}
	].
	(dialogName = 'motors') ifTrue: [
		^ {NXTMotor . NXTDCMotor}
	].

	^ #()! !


!NXTConfiguration class methodsFor: 'block specs' stamp: 'CWB 1/28/2012 21:04'!
deviceBlockSpecs
"Override this to supply block specs that should always be active for this device."

	| specs inapplicableSpecs |

	specs _ #(
		'sound'
			"Commented out lines are pending development, Remove them from final if still commented out."
			"('play sound %S'				-	playSound:)"
			"('play sound %S until done'		s	doPlaySoundAndWait)"
			"-"
			('rest for %n beats'				t	rest:elapsed:from: 0.5)
			-
			('play note %N for %n beats'	t	noteOn:duration:elapsed:from: 60 0.5)
			('set instrument to %I'			- 	nxtMidiInstrument: 1)
			('instrument'					r	nxtMidiInstrument)
			-
			('change volume by %n'		- 	nxtChangeVolumeBy: -10)
			('set volume to %n%'			- 	nxtSetVolumeTo: 100)
			('volume'						r 	nxtVolume)
			-
			('change tempo by %n'			- 	nxtChangeTempoBy: 20)
			('set tempo to %n bpm'			-	nxtSetTempoTo: 60)
			('tempo'							r 	nxtTempo)
		'looks'
			('print %s'						-	nxtPrint: 'Hello')
		'sensing'	
			('battery voltage'				r	nxtBatteryVoltage)
			-
			('%H sensor value'			r	sensor: '(remote)')
			"('sensor %h?'				b	sensorPressed: 'button pressed')"
			-
			('reset timer'					-	timerReset)
			('timer'							r	timer)
	).

	inapplicableSpecs _ #(
		('when %k key pressed' #K #-)
		('when %m clicked' #M #-)
		('broadcast %e and wait' #s #doBroadcastAndWait)
	).

	
	"return all specs that aren't inapplicable"
	^ specs, self standardBlockSpecs 
		reject: [:spec | inapplicableSpecs includes: spec ]. 


! !
